% =============================================== %
% returns a cell array containing the monomials
% and the coefficients of the basis functions
% =============================================== %

function [monomials, B] = assemble_functions()

% Output:       a (21 x 6) cell array and a (21 x 21) matrix.
% 
% monomials:    in each column, a monomial up to degree 5
%               is stored, in the order f, fx, fy, fxx, fxy, fyy.
%
% B:            each row B(i, :) contains the coefficients of the basis
%               function phi_i with respect to these monomials
%               The numbering corresponds to the one in defelement.com,
%               with the difference of starting at 1 instead of 0.

monomials = cell(21, 6);

monomials{ 1, 1} = @(x) 1;
monomials{ 1, 2} = @(x) 0;
monomials{ 1, 3} = @(x) 0;
monomials{ 1, 4} = @(x) 0;
monomials{ 1, 5} = @(x) 0;
monomials{ 1, 6} = @(x) 0;

%

monomials{ 2, 1} = @(x) x(1);
monomials{ 2, 2} = @(x) 1;
monomials{ 2, 3} = @(x) 0;
monomials{ 2, 4} = @(x) 0;
monomials{ 2, 5} = @(x) 0;
monomials{ 2, 6} = @(x) 0;

monomials{ 3, 1} = @(x) x(2);
monomials{ 3, 2} = @(x) 0;
monomials{ 3, 3} = @(x) 1;
monomials{ 3, 4} = @(x) 0;
monomials{ 3, 5} = @(x) 0;
monomials{ 3, 6} = @(x) 0;

%

monomials{ 4, 1} = @(x) x(1)^2;
monomials{ 4, 2} = @(x) 2*x(1);
monomials{ 4, 3} = @(x) 0;
monomials{ 4, 4} = @(x) 2;
monomials{ 4, 5} = @(x) 0;
monomials{ 4, 6} = @(x) 0;

monomials{ 5, 1} = @(x) x(1)*x(2);
monomials{ 5, 2} = @(x) x(2);
monomials{ 5, 3} = @(x) x(1);
monomials{ 5, 4} = @(x) 0;
monomials{ 5, 5} = @(x) 1;
monomials{ 5, 6} = @(x) 0;

monomials{ 6, 1} = @(x) x(2)^2;
monomials{ 6, 2} = @(x) 0;
monomials{ 6, 3} = @(x) 2*x(2);
monomials{ 6, 4} = @(x) 0;
monomials{ 6, 5} = @(x) 0;
monomials{ 6, 6} = @(x) 2;

% 

monomials{ 7, 1} = @(x) x(1)^3;
monomials{ 7, 2} = @(x) 3*x(1)^2;
monomials{ 7, 3} = @(x) 0;
monomials{ 7, 4} = @(x) 6*x(1);
monomials{ 7, 5} = @(x) 0;
monomials{ 7, 6} = @(x) 0;

monomials{ 8, 1} = @(x) x(1)^2*x(2);
monomials{ 8, 2} = @(x) 2*x(1)*x(2);
monomials{ 8, 3} = @(x) x(1)^2;
monomials{ 8, 4} = @(x) 2*x(2);
monomials{ 8, 5} = @(x) 2*x(1);
monomials{ 8, 6} = @(x) 0;

monomials{ 9, 1} = @(x) x(1)*x(2)^2;
monomials{ 9, 2} = @(x) x(2)^2;
monomials{ 9, 3} = @(x) 2*x(1)*x(2);
monomials{ 9, 4} = @(x) 0;
monomials{ 9, 5} = @(x) 2*x(2);
monomials{ 9, 6} = @(x) 2*x(1);

monomials{10, 1} = @(x) x(2)^3;
monomials{10, 2} = @(x) 0;
monomials{10, 3} = @(x) 3*x(2)^2;
monomials{10, 4} = @(x) 0;
monomials{10, 5} = @(x) 0;
monomials{10, 6} = @(x) 6*x(2);

% 

monomials{11, 1} = @(x) x(1)^4;
monomials{11, 2} = @(x) 4*x(1)^3;
monomials{11, 3} = @(x) 0;
monomials{11, 4} = @(x) 12*x(1)^2;
monomials{11, 5} = @(x) 0;
monomials{11, 6} = @(x) 0;

monomials{12, 1} = @(x) x(1)^3*x(2);
monomials{12, 2} = @(x) 3*x(1)^2*x(2);
monomials{12, 3} = @(x) x(1)^3;
monomials{12, 4} = @(x) 6*x(1)*x(2);
monomials{12, 5} = @(x) 3*x(1)^2;
monomials{12, 6} = @(x) 0;

monomials{13, 1} = @(x) x(1)^2*x(2)^2;
monomials{13, 2} = @(x) 2*x(1)*x(2)^2;
monomials{13, 3} = @(x) 2*x(1)^2*x(2);
monomials{13, 4} = @(x) 2*x(2)^2;
monomials{13, 5} = @(x) 4*x(1)*x(2);
monomials{13, 6} = @(x) 2*x(1)^2;

monomials{14, 1} = @(x) x(1)*x(2)^3;
monomials{14, 2} = @(x) x(2)^3;
monomials{14, 3} = @(x) 3*x(1)*x(2)^2;
monomials{14, 4} = @(x) 0;
monomials{14, 5} = @(x) 3*x(2)^2;
monomials{14, 6} = @(x) 6*x(1)*x(2);

monomials{15, 1} = @(x) x(2)^4;
monomials{15, 2} = @(x) 0;
monomials{15, 3} = @(x) 4*x(2)^3;
monomials{15, 4} = @(x) 0;
monomials{15, 5} = @(x) 0;
monomials{15, 6} = @(x) 12*x(2)^2;

% 

monomials{16, 1} = @(x) x(1)^5;
monomials{16, 2} = @(x) 5*x(1)^4;
monomials{16, 3} = @(x) 0;
monomials{16, 4} = @(x) 20*x(1)^3;
monomials{16, 5} = @(x) 0;
monomials{16, 6} = @(x) 0;

monomials{17, 1} = @(x) x(1)^4*x(2);
monomials{17, 2} = @(x) 4*x(1)^3*x(2);
monomials{17, 3} = @(x) x(1)^4;
monomials{17, 4} = @(x) 12*x(1)^2*x(2);
monomials{17, 5} = @(x) 4*x(1)^3;
monomials{17, 6} = @(x) 0;

monomials{18, 1} = @(x) x(1)^3*x(2)^2;
monomials{18, 2} = @(x) 3*x(1)^2*x(2)^2;
monomials{18, 3} = @(x) 2*x(1)^3*x(2);
monomials{18, 4} = @(x) 6*x(1)*x(2)^2;
monomials{18, 5} = @(x) 6*x(1)^2*x(2);
monomials{18, 6} = @(x) 2*x(1)^3;

monomials{19, 1} = @(x) x(1)^2*x(2)^3;
monomials{19, 2} = @(x) 2*x(1)*x(2)^3;
monomials{19, 3} = @(x) 3*x(1)^2*x(2)^2;
monomials{19, 4} = @(x) 2*x(2)^3;
monomials{19, 5} = @(x) 6*x(1)*x(2)^2;
monomials{19, 6} = @(x) 6*x(1)^2*x(2);

monomials{20, 1} = @(x) x(1)*x(2)^4;
monomials{20, 2} = @(x) x(2)^4;
monomials{20, 3} = @(x) 4*x(1)*x(2)^3;
monomials{20, 4} = @(x) 0;
monomials{20, 5} = @(x) 4*x(2)^3;
monomials{20, 6} = @(x) 12*x(1)*x(2)^2;

monomials{21, 1} = @(x) x(2)^5;
monomials{21, 2} = @(x) 0;
monomials{21, 3} = @(x) 5*x(2)^4;
monomials{21, 4} = @(x) 0;
monomials{21, 5} = @(x) 0;
monomials{21, 6} = @(x) 20*x(2)^3;



% store the coefficient matrix: in each row, the coefficients of the j-th
% basis functions are stored, so that phi_j = A*monomials
B = zeros(21);

B( 1,  1) =   1;
B( 1,  7) = -10;
B( 1, 10) = -10;
B( 1, 11) =  15;
B( 1, 13) = -30;
B( 1, 15) =  15;
B( 1, 16) = - 6;
B( 1, 18) =  30;
B( 1, 19) =  30;
B( 1, 21) = - 6;

B( 2,  2) =   1;
B( 2,  7) = - 6;
B( 2,  9) = -11;
B( 2, 11) =   8;
B( 2, 13) =  10;
B( 2, 14) =  18;
B( 2, 16) = - 3;
B( 2, 18) =   1;
B( 2, 19) = -10;
B( 2, 20) = - 8;

B( 3,  3) =   1;
B( 3,  8) = -11;
B( 3, 10) =  -6;
B( 3, 12) =  18;
B( 3, 13) =  10;
B( 3, 15) =   8;
B( 3, 17) =  -8;
B( 3, 18) = -10;
B( 3, 19) =   1;
B( 3, 21) =  -3;

B( 4,  4) =   1;
B( 4,  7) =  -3;
B( 4, 11) =   3;
B( 4, 13) =  -3;
B( 4, 16) =  -1;
B( 4, 18) =   3;
B( 4, 19) =   2;
B( 4,  :) = 0.5*B(4, :);

B( 5,  5) =   1;
B( 5,  8) =  -4;
B( 5,  9) =  -4;
B( 5, 12) =   5;
B( 5, 13) =  10;
B( 5, 14) =   5;
B( 5, 17) =  -2;
B( 5, 18) =  -6;
B( 5, 19) =  -6;
B( 5, 20) =  -2;

B( 6,  6) =   1;
B( 6, 10) =  -3;
B( 6, 13) =  -3;
B( 6, 15) =   3;
B( 6, 18) =   2;
B( 6, 19) =   3;
B( 6, 21) =  -1;
B( 6,  :) = 0.5*B(6, :);

B( 7,  7) =  10;
B( 7, 11) = -15;
B( 7, 13) =  15;
B( 7, 16) =   6;
B( 7, 18) = -15;
B( 7, 19) = -15;

B( 8,  7) =  -8;
B( 8, 11) =  14;
B( 8, 13) =  -7;
B( 8, 16) =  -6;
B( 8, 18) =   7;
B( 8, 19) =   7;
B( 8,  :) = 0.5*B(8, :);

B( 9,  8) = -10;
B( 9, 12) =  28;
B( 9, 13) =  37;
B( 9, 17) = -16;
B( 9, 18) = -37;
B( 9, 19) = -27;
B( 9,  :) = 0.5*B(9, :);

B(10,  7) =   2;
B(10, 11) =  -4;
B(10, 13) =   1;
B(10, 16) =   2;
B(10, 18) =  -1;
B(10, 19) =  -1;
B(10,  :) = 0.25*B(10, :);

B(11,  8) =   2;
B(11, 12) =  -6;
B(11, 13) =  -7;
B(11, 17) =   4;
B(11, 18) =   7;
B(11, 19) =   5;
B(11,  :) = 0.5*B(11, :);

B(12, 13) =   5;
B(12, 18) =  -3;
B(12, 19) =  -5;
B(12,  :) = 0.25*B(12, :);

B(13, 10) =  10;
B(13, 13) =  15;
B(13, 15) = -15;
B(13, 18) = -15;
B(13, 19) = -15;
B(13, 21) =   6;

B(14,  9) = -10;
B(14, 13) =  37;
B(14, 14) =  28;
B(14, 18) = -27;
B(14, 19) = -37;
B(14, 20) = -16;
B(14,  :) = 0.5*B(14, :);

B(15, 10) =  -8;
B(15, 13) =  -7;
B(15, 15) =  14;
B(15, 18) =   7;
B(15, 19) =   7;
B(15, 21) =  -6;
B(15,  :) = 0.5*B(15, :);

B(16, 13) =   5;
B(16, 18) =  -5;
B(16, 19) =  -3;
B(16,  :) = 0.25*B(16, :);

B(17,  9) =   2;
B(17, 13) =  -7;
B(17, 14) =  -6;
B(17, 18) =   5;
B(17, 19) =   7;
B(17, 20) =   4;
B(17,  :) = 0.5*B(17, :);

B(18, 10) =   2;
B(18, 13) =   1;
B(18, 15) =  -4;
B(18, 18) =  -1;
B(18, 19) =  -1;
B(18, 21) =   2;
B(18,  :) = 0.25*B(18,  :);

B(19, 13) =   1;
B(19, 18) =  -1;
B(19, 19) =  -1;
B(19,  :) = -8*sqrt(2)*B(19, :);

B(20,  9) =  -1;
B(20, 13) =   2;
B(20, 14) =   2;
B(20, 18) =  -1;
B(20, 19) =  -2;
B(20, 20) =  -1;
B(20,  :) = 16*B(20, :);

B(21,  8) =   1;
B(21, 12) =  -2;
B(21, 13) =  -2;
B(21, 17) =   1;
B(21, 18) =   2;
B(21, 19) =   1;
B(21,  :) = -16*B(21, :);





end


